//!
//!

use super::models;
use crate::repositories;
use beteran_common_rust as bcr;
use diesel::{result::Error, sql_query, RunQueryDsl};
use std::fmt::Write;

static MEMBER_CHILDREN_COUNT_QUERY: &str = "
WITH RECURSIVE cte AS (
  SELECT 
    id, parent_member_id, 1::integer recursion_level
  FROM
    members
  WHERE 
    parent_member_id = $1
  UNION ALL
  SELECT 
    m.id, m.parent_member_id, cte.recursion_level +1
  FROM
    members m
    JOIN cte ON cte.id = m.parent_member_id 
  )
  SELECT 
    count(recursion_level) AS count_of_children
  FROM 
    cte 
  group by 
    recursion_level 
  order by 
    recursion_level asc
";

static MEMBER_PARENTS_QUERY: &str = "
WITH RECURSIVE rec AS (

  SELECT
    m.id                     as m_id,
    m.site_id                as m_site_id,
    m.member_class_id        as m_member_class_id,  
    m.member_level_id        as m_member_level_id,    
    m.username               as m_username,  
    m.password               as m_password,  
    m.nickname               as m_nickname,
    m.mobile_phone_number    as m_mobile_phone_number,
    m.state                  as m_state,
    m.state_changed_at       as m_state_changed_at,
    m.parent_member_id       as m_parent_member_id,
    m.child_member_count     as m_child_member_count,
    m.last_signined_ip       as m_last_signined_ip,
    m.last_signined_at       as m_last_signined_at,
    m.created_at             as m_created_at,
    m.updated_at             as m_updated_at,
    m.deleted_at             as m_deleted_at,    
  
    s.id           as s_id,          
    s.url          as s_url,         
    s.name         as s_name,         
    s.path         as s_path,         
    s.show         as s_show,         
    s.can_use      as s_can_use,         
    s.memo         as s_memo,         
    s.expires_at   as s_expires_at,  
    s.created_at   as s_created_at,  
    s.updated_at   as s_updated_at,  
  
    mc.id           as mc_id,        
    mc.parent_id    as mc_parent_id, 
    mc.name         as mc_name,      
    mc.show         as mc_show,      
    mc.created_at   as mc_created_at,
    mc.updated_at   as mc_updated_at,
  
    ml.id           as ml_id,
    ml.name         as ml_name,
    ml.show         as ml_show,
    ml.sort_order   as ml_sort_order,
    ml.created_at   as ml_created_at,
    ml.updated_at   as ml_updated_at,
  
    mgs.id                                as mgs_id,
    mgs.member_id                         as mgs_member_id,
    mgs.can_bet_casino                    as mgs_can_bet_casino,
    mgs.can_bet_slot                      as mgs_can_bet_slot,
    mgs.can_bet_powerball                 as mgs_can_bet_powerball,
    mgs.can_bet_powerladder               as mgs_can_bet_powerladder,
    mgs.can_bet_eos                       as mgs_can_bet_eos,
    mgs.can_bet_bogglepowerball           as mgs_can_bet_bogglepowerball,
    mgs.created_at                        as mgs_created_at,
    mgs.updated_at                        as mgs_updated_at,
  
    mss.id                                as mss_id,
    mss.member_id                         as mss_member_id,
    mss.can_exchange                      as mss_can_exchange,
    mss.can_first_charge_comp             as mss_can_first_charge_comp,
    mss.can_per_charge_comp               as mss_can_per_charge_comp,
    mss.manual_payment_type_for_partner   as mss_manual_payment_type_for_partner,
    mss.settlement_type                   as mss_settlement_type,
    mss.rate_casino                       as mss_rate_casino,
    mss.rate_casino_loosing               as mss_rate_casino_loosing,
    mss.rate_casino_bacara                as mss_rate_casino_bacara,
    mss.rate_casino_roulette              as mss_rate_casino_roulette,
    mss.rate_casino_dragon_tiger          as mss_rate_casino_dragon_tiger,
    mss.rate_slot                         as mss_rate_slot,
    mss.rate_slot_loosing                 as mss_rate_slot_loosing,
    mss.rate_powerball_single             as mss_rate_powerball_single,
    mss.rate_powerball_combo              as mss_rate_powerball_combo,
    mss.rate_powerladder_single           as mss_rate_powerladder_single,
    mss.rate_powerladder_combo            as mss_rate_powerladder_combo,
    mss.rate_eos_single                   as mss_rate_eos_single,
    mss.rate_eos_combo                    as mss_rate_eos_combo,
    mss.rate_bogglepowerball_single       as mss_rate_bogglepowerball_single,
    mss.rate_bogglepowerball_combo        as mss_rate_bogglepowerball_combo,
    mss.created_at                        as mss_created_at,
    mss.updated_at                        as mss_updated_at,
  
    mba.id                 as mba_id,
    mba.member_id          as mba_member_id,
    mba.bank_id            as mba_bank_id,
    mba.name               as mba_name,
    mba.account_number     as mba_account_number,
    mba.exchange_password  as mba_exchange_password,
    mba.memo               as mba_memo,
    mba.created_at         as mba_created_at,
    mba.updated_at         as mba_updated_at,
  
    mba_b.id              as mba_b_id,
    mba_b.name            as mba_b_name,
    mba_b.sort_order      as mba_b_sort_order,
    mba_b.show            as mba_b_show,
    mba_b.can_use         as mba_b_can_use,
    mba_b.memo            as mba_b_memo,
    mba_b.created_at      as mba_b_created_at,
    mba_b.updated_at      as mba_b_updated_at,
  
    1 AS level 
  FROM 
    members m
    INNER JOIN sites s
      ON s.id = m.site_id
    INNER JOIN member_classes mc
      ON mc.id = m.member_class_id
    INNER JOIN member_levels ml
      ON ml.id = m.member_level_id
    LEFT OUTER JOIN member_settlement_settings mss
      ON mss.member_id = m.id
    LEFT OUTER JOIN member_game_settings mgs
      ON mgs.member_id = m.id
    LEFT OUTER JOIN member_bank_accounts mba
      ON mba.member_id = m.id
    LEFT OUTER JOIN banks mba_b
      ON mba_b.id = mba.bank_id
  
  WHERE
    m.id = $1
  
  UNION ALL 
  
  SELECT
    c_m.id                     as m_id,
    c_m.site_id                as m_site_id,
    c_m.member_class_id        as m_member_class_id,  
    c_m.member_level_id        as m_member_level_id,    
    c_m.username               as m_username,  
    c_m.password               as m_password,  
    c_m.nickname               as m_nickname,
    c_m.mobile_phone_number    as m_mobile_phone_number,
    c_m.state                  as m_state,
    c_m.state_changed_at       as m_state_changed_at,
    c_m.parent_member_id       as m_parent_member_id,
    c_m.child_member_count     as m_child_member_count,
    c_m.last_signined_ip       as m_last_signined_ip,
    c_m.last_signined_at       as m_last_signined_at,
    c_m.created_at             as m_created_at,
    c_m.updated_at             as m_updated_at,
    c_m.deleted_at             as m_deleted_at,    
  
    c_s.id           as s_id,          
    c_s.url          as s_url,         
    c_s.name         as s_name,         
    c_s.path         as s_path,         
    c_s.show         as s_show,         
    c_s.can_use      as s_can_use,         
    c_s.memo         as s_memo,         
    c_s.expires_at   as s_expires_at,  
    c_s.created_at   as s_created_at,  
    c_s.updated_at   as s_updated_at,  
  
    c_mc.id           as mc_id,        
    c_mc.parent_id    as mc_parent_id, 
    c_mc.name         as mc_name,      
    c_mc.show         as mc_show,      
    c_mc.created_at   as mc_created_at,
    c_mc.updated_at   as mc_updated_at,
  
    c_ml.id           as ml_id,
    c_ml.name         as ml_name,
    c_ml.show         as ml_show,
    c_ml.sort_order   as ml_sort_order,
    c_ml.created_at   as ml_created_at,
    c_ml.updated_at   as ml_updated_at,
  
    c_mgs.id                                as mgs_id,
    c_mgs.member_id                         as mgs_member_id,
    c_mgs.can_bet_casino                    as mgs_can_bet_casino,
    c_mgs.can_bet_slot                      as mgs_can_bet_slot,
    c_mgs.can_bet_powerball                 as mgs_can_bet_powerball,
    c_mgs.can_bet_powerladder               as mgs_can_bet_powerladder,
    c_mgs.can_bet_eos                       as mgs_can_bet_eos,
    c_mgs.can_bet_bogglepowerball           as mgs_can_bet_bogglepowerball,
    c_mgs.created_at                        as mgs_created_at,
    c_mgs.updated_at                        as mgs_updated_at,
  
    c_mss.id                                as mss_id,
    c_mss.member_id                         as mss_member_id,
    c_mss.can_exchange                      as mss_can_exchange,
    c_mss.can_first_charge_comp             as mss_can_first_charge_comp,
    c_mss.can_per_charge_comp               as mss_can_per_charge_comp,
    c_mss.manual_payment_type_for_partner   as mss_manual_payment_type_for_partner,
    c_mss.settlement_type                   as mss_settlement_type,
    c_mss.rate_casino                       as mss_rate_casino,
    c_mss.rate_casino_loosing               as mss_rate_casino_loosing,
    c_mss.rate_casino_bacara                as mss_rate_casino_bacara,
    c_mss.rate_casino_roulette              as mss_rate_casino_roulette,
    c_mss.rate_casino_dragon_tiger          as mss_rate_casino_dragon_tiger,
    c_mss.rate_slot                         as mss_rate_slot,
    c_mss.rate_slot_loosing                 as mss_rate_slot_loosing,
    c_mss.rate_powerball_single             as mss_rate_powerball_single,
    c_mss.rate_powerball_combo              as mss_rate_powerball_combo,
    c_mss.rate_powerladder_single           as mss_rate_powerladder_single,
    c_mss.rate_powerladder_combo            as mss_rate_powerladder_combo,
    c_mss.rate_eos_single                   as mss_rate_eos_single,
    c_mss.rate_eos_combo                    as mss_rate_eos_combo,
    c_mss.rate_bogglepowerball_single       as mss_rate_bogglepowerball_single,
    c_mss.rate_bogglepowerball_combo        as mss_rate_bogglepowerball_combo,
    c_mss.created_at                        as mss_created_at,
    c_mss.updated_at                        as mss_updated_at,
  
    c_mba.id                 as mba_id,
    c_mba.member_id          as mba_member_id,
    c_mba.bank_id            as mba_bank_id,
    c_mba.name               as mba_name,
    c_mba.account_number     as mba_account_number,
    c_mba.exchange_password  as mba_exchange_password,
    c_mba.memo               as mba_memo,
    c_mba.created_at         as mba_created_at,
    c_mba.updated_at         as mba_updated_at,
  
    c_mba_b.id              as mba_b_id,
    c_mba_b.name            as mba_b_name,
    c_mba_b.sort_order      as mba_b_sort_order,
    c_mba_b.show            as mba_b_show,
    c_mba_b.can_use         as mba_b_can_use,
    c_mba_b.memo            as mba_b_memo,
    c_mba_b.created_at      as mba_b_created_at,
    c_mba_b.updated_at      as mba_b_updated_at, 
  
    (p_m.level+1) 
  FROM 
    members c_m
    INNER JOIN sites c_s
      ON c_s.id = c_m.site_id
    INNER JOIN member_classes c_mc
      ON c_mc.id = c_m.member_class_id
    INNER JOIN member_levels c_ml
      ON c_ml.id = c_m.member_level_id
    LEFT OUTER JOIN member_settlement_settings c_mss
      ON c_mss.member_id = c_m.id
    LEFT OUTER JOIN member_game_settings c_mgs
      ON c_mgs.member_id = c_m.id
    LEFT OUTER JOIN member_bank_accounts c_mba
      ON c_mba.member_id = c_m.id
    LEFT OUTER JOIN banks c_mba_b
      ON c_mba_b.id = c_mba.bank_id
    JOIN rec p_m
      ON c_m.id = p_m.m_parent_member_id
  )
  SELECT 
    DISTINCT 
    m_id,
    m_site_id,
    m_member_class_id,  
    m_member_level_id,    
    m_username,  
    m_password,  
    m_nickname,
    m_mobile_phone_number,
    m_state,
    m_state_changed_at,
    m_parent_member_id,
    m_child_member_count,
    m_last_signined_ip,
    m_last_signined_at,
    m_created_at,
    m_updated_at,
    m_deleted_at,    
  
    s_id,          
    s_url,         
    s_name,         
    s_path,         
    s_show,         
    s_can_use,       
    s_memo,         
    s_expires_at,  
    s_created_at,  
    s_updated_at,  
  
    mc_id,        
    mc_parent_id, 
    mc_name,      
    mc_show,      
    mc_created_at,
    mc_updated_at,
  
    ml_id,
    ml_name,
    ml_show,
    ml_sort_order,
    ml_created_at,
    ml_updated_at,
  
    mgs_id,
    mgs_member_id,
    mgs_can_bet_casino,
    mgs_can_bet_slot,
    mgs_can_bet_powerball,
    mgs_can_bet_powerladder,
    mgs_can_bet_eos,
    mgs_can_bet_bogglepowerball,
    mgs_created_at,
    mgs_updated_at,
  
    mss_id,
    mss_member_id,
    mss_can_exchange,
    mss_can_first_charge_comp,
    mss_can_per_charge_comp,
    mss_manual_payment_type_for_partner,
    mss_settlement_type,
    mss_rate_casino,
    mss_rate_casino_loosing,
    mss_rate_casino_bacara,
    mss_rate_casino_roulette,
    mss_rate_casino_dragon_tiger,
    mss_rate_slot,
    mss_rate_slot_loosing,
    mss_rate_powerball_single,
    mss_rate_powerball_combo,
    mss_rate_powerladder_single,
    mss_rate_powerladder_combo,
    mss_rate_eos_single,
    mss_rate_eos_combo,
    mss_rate_bogglepowerball_single,
    mss_rate_bogglepowerball_combo,
    mss_created_at,
    mss_updated_at,
  
    mba_id,
    mba_member_id,
    mba_bank_id,
    mba_name,
    mba_account_number,
    mba_exchange_password,
    mba_memo,
    mba_created_at,
    mba_updated_at,
  
    mba_b_id,
    mba_b_name,
    mba_b_sort_order,
    mba_b_show,
    mba_b_can_use,
    mba_b_memo,
    mba_b_created_at,
    mba_b_updated_at, 
  
    level 
  FROM 
    rec 
  ORDER BY 
    level DESC
";

static MEMBER_COUNT_QUERY: &str = "
SELECT
  COUNT(m.id)
FROM members as m
  INNER JOIN sites s
    ON s.id = m.site_id
  INNER JOIN member_classes mc
    ON mc.id = m.member_class_id
  INNER JOIN member_levels ml
    ON ml.id = m.member_level_id
  LEFT OUTER JOIN member_settlement_settings mss
    ON mss.member_id = m.id
  LEFT OUTER JOIN member_game_settings mgs
    ON mgs.member_id = m.id
  LEFT OUTER JOIN member_bank_accounts mba
    ON mba.member_id = m.id
  LEFT OUTER JOIN banks mba_b
    ON mba_b.id = mba.bank_id
";

static MEMBER_QUERY: &str = "
    SELECT
      m.id                     as m_id,
      m.site_id                as m_site_id,
      m.member_class_id        as m_member_class_id,  
      m.member_level_id        as m_member_level_id,    
      m.username               as m_username,  
      m.password               as m_password,  
      m.nickname               as m_nickname,
      m.mobile_phone_number    as m_mobile_phone_number,
      m.state                  as m_state,
      m.state_changed_at       as m_state_changed_at,
      m.parent_member_id       as m_parent_member_id,
      m.child_member_count     as m_child_member_count,
      m.last_signined_ip       as m_last_signined_ip,
      m.last_signined_at       as m_last_signined_at,
      m.created_at             as m_created_at,
      m.updated_at             as m_updated_at,
      m.deleted_at             as m_deleted_at,    

      s.id           as s_id,          
      s.url          as s_url,         
      s.name         as s_name,         
      s.path         as s_path,         
      s.show         as s_show,         
      s.can_use      as s_can_use,         
      s.memo         as s_memo,         
      s.expires_at   as s_expires_at,  
      s.created_at   as s_created_at,  
      s.updated_at   as s_updated_at,  
      
      mc.id           as mc_id,        
      mc.parent_id    as mc_parent_id, 
      mc.name         as mc_name,      
      mc.show         as mc_show,      
      mc.created_at   as mc_created_at,
      mc.updated_at   as mc_updated_at,
      
      ml.id           as ml_id,
      ml.name         as ml_name,
      ml.show         as ml_show,
      ml.sort_order   as ml_sort_order,
      ml.created_at   as ml_created_at,
      ml.updated_at   as ml_updated_at,
      
      mgs.id                                as mgs_id,
      mgs.member_id                         as mgs_member_id,
      mgs.can_bet_casino                    as mgs_can_bet_casino,
      mgs.can_bet_slot                      as mgs_can_bet_slot,
      mgs.can_bet_powerball                 as mgs_can_bet_powerball,
      mgs.can_bet_powerladder               as mgs_can_bet_powerladder,
      mgs.can_bet_eos                       as mgs_can_bet_eos,
      mgs.can_bet_bogglepowerball           as mgs_can_bet_bogglepowerball,
      mgs.created_at                        as mgs_created_at,
      mgs.updated_at                        as mgs_updated_at,
    
      mss.id                                as mss_id,
      mss.member_id                         as mss_member_id,
      mss.can_exchange                      as mss_can_exchange,
      mss.can_first_charge_comp             as mss_can_first_charge_comp,
      mss.can_per_charge_comp               as mss_can_per_charge_comp,
      mss.manual_payment_type_for_partner   as mss_manual_payment_type_for_partner,
      mss.settlement_type                   as mss_settlement_type,
      mss.rate_casino                       as mss_rate_casino,
      mss.rate_casino_loosing               as mss_rate_casino_loosing,
      mss.rate_casino_bacara                as mss_rate_casino_bacara,
      mss.rate_casino_roulette              as mss_rate_casino_roulette,
      mss.rate_casino_dragon_tiger          as mss_rate_casino_dragon_tiger,
      mss.rate_slot                         as mss_rate_slot,
      mss.rate_slot_loosing                 as mss_rate_slot_loosing,
      mss.rate_powerball_single             as mss_rate_powerball_single,
      mss.rate_powerball_combo              as mss_rate_powerball_combo,
      mss.rate_powerladder_single           as mss_rate_powerladder_single,
      mss.rate_powerladder_combo            as mss_rate_powerladder_combo,
      mss.rate_eos_single                   as mss_rate_eos_single,
      mss.rate_eos_combo                    as mss_rate_eos_combo,
      mss.rate_bogglepowerball_single       as mss_rate_bogglepowerball_single,
      mss.rate_bogglepowerball_combo        as mss_rate_bogglepowerball_combo,
      mss.created_at                        as mss_created_at,
      mss.updated_at                        as mss_updated_at,

      mba.id                 as mba_id,
      mba.member_id          as mba_member_id,
      mba.bank_id            as mba_bank_id,
      mba.name               as mba_name,
      mba.account_number     as mba_account_number,
      mba.exchange_password  as mba_exchange_password,
      mba.memo               as mba_memo,
      mba.created_at         as mba_created_at,
      mba.updated_at         as mba_updated_at,

      mba_b.id              as mba_b_id,
      mba_b.name            as mba_b_name,
      mba_b.sort_order      as mba_b_sort_order,
      mba_b.show            as mba_b_show,
      mba_b.can_use         as mba_b_can_use,
      mba_b.memo            as mba_b_memo,
      mba_b.created_at      as mba_b_created_at,
      mba_b.updated_at      as mba_b_updated_at
    
    FROM members as m
      INNER JOIN sites s
        ON s.id = m.site_id
      INNER JOIN member_classes mc
        ON mc.id = m.member_class_id
      INNER JOIN member_levels ml
        ON ml.id = m.member_level_id
      LEFT OUTER JOIN member_settlement_settings mss
        ON mss.member_id = m.id
      LEFT OUTER JOIN member_game_settings mgs
        ON mgs.member_id = m.id
      LEFT OUTER JOIN member_bank_accounts mba
        ON mba.member_id = m.id
      LEFT OUTER JOIN banks mba_b
        ON mba_b.id = mba.bank_id
";

pub struct Composition {}

impl std::fmt::Debug for Composition {
  fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
    f.debug_struct("Composition of members").finish()
  }
}

impl Default for Composition {
  fn default() -> Self {
    Self::new()
  }
}

impl Composition {
  ///
  pub fn new() -> Composition {
    Composition {}
  }

  ///
  pub fn select(
    &self,
    conn: &diesel::PgConnection,
    id: uuid::Uuid,
  ) -> Result<Option<models::MemberModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      m.id = $1
    ",
      MEMBER_QUERY
    );

    let _m = match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(id)
      .get_result::<models::_MemberModel>(conn)
    {
      Ok(m) => m,
      Err(e) => match e {
        diesel::result::Error::NotFound => {
          return Ok(None);
        }
        _ => {
          return Err(e);
        }
      },
    };

    Ok(Some(models::from_member_model(
      &_m,
      self.select_parent(conn, _m.parent_member_id)?,
      self.select_children_count(conn, _m.id)?,
    )))
  }

  ///
  pub fn select_by_username(
    &self,
    conn: &diesel::PgConnection,
    username: &str,
  ) -> Result<Option<models::MemberModel>, Error> {
    let query = format!(
      "
    {}
    WHERE
      m.username = $1
    ",
      MEMBER_QUERY
    );

    let _m = match sql_query(query)
      .bind::<diesel::sql_types::Text, _>(username)
      .get_result::<models::_MemberModel>(conn)
    {
      Ok(m) => m,
      Err(e) => match e {
        diesel::result::Error::NotFound => {
          return Ok(None);
        }
        _ => {
          return Err(e);
        }
      },
    };

    Ok(Some(models::from_member_model(
      &_m,
      self.select_parent(conn, _m.parent_member_id)?,
      self.select_children_count(conn, _m.id)?,
    )))
  }

  ///
  pub fn select_children_count(
    &self,
    conn: &diesel::PgConnection,
    parent_member_id: uuid::Uuid,
  ) -> Result<Vec<i64>, Error> {
    use diesel::sql_types::BigInt;

    #[derive(QueryableByName)]
    struct Count {
      #[sql_type = "BigInt"]
      count_of_children: i64,
    }

    let query = format!(
      "
    {}
    ",
      MEMBER_CHILDREN_COUNT_QUERY
    );

    let count_of_children = match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(parent_member_id)
      .get_results::<Count>(conn)
    {
      Ok(m) => m,
      Err(e) => match e {
        diesel::result::Error::NotFound => vec![],
        _ => {
          return Err(e);
        }
      },
    };

    if count_of_children.is_empty() {
      return Ok(vec![]);
    }

    let mut p: Vec<i64> = vec![];
    for c in count_of_children {
      p.push(c.count_of_children);
    }

    Ok(p)
  }

  ///
  pub fn select_parent(
    &self,
    conn: &diesel::PgConnection,
    parent_member_id: Option<uuid::Uuid>,
  ) -> Result<Option<models::MemberModel>, Error> {
    let parent_member_id = match parent_member_id {
      Some(i) => i,
      None => {
        return Ok(None);
      }
    };

    let query = format!(
      "
    {}
    ",
      MEMBER_PARENTS_QUERY
    );

    let members = match sql_query(query)
      .bind::<diesel::sql_types::Uuid, _>(parent_member_id)
      .get_results::<models::_MemberModel>(conn)
    {
      Ok(m) => m,
      Err(e) => match e {
        diesel::result::Error::NotFound => vec![],
        _ => {
          return Err(e);
        }
      },
    };

    if members.is_empty() {
      return Ok(None);
    }

    let mut p: Option<models::MemberModel> = None;
    for m in members {
      p = Some(models::from_member_model(
        &m,
        p,
        self.select_children_count(conn, m.id)?,
      ));
    }

    Ok(p)
  }

  fn get_pagination(
    &self,
    find_all: &repositories::member::models::FindAll,
  ) -> Result<String, Error> {
    let mut query_pagination = String::new();

    if let Some(pagination) = &find_all.pagination {
      let page = match pagination.page {
        Some(v) => {
          if v < 1 {
            1
          } else {
            v
          }
        }
        None => 1,
      };
      if let Some(page_size) = pagination.page_size {
        write!(
          &mut query_pagination,
          " OFFSET {} LIMIT {} ",
          ((page - 1) * page_size) as i64,
          page_size as i64
        )
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }

    Ok(query_pagination)
  }

  fn get_ordery_by(
    &self,
    find_all: &repositories::member::models::FindAll,
  ) -> Result<String, Error> {
    let mut query_ordery_by = String::new();

    if let Some(sorts) = &find_all.sorts {
      for s in sorts {
        if !query_ordery_by.is_empty() {
          write!(&mut query_ordery_by, " , ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }

        let (property, sort) = match s {
          bcr::pagination::Sort::ASC(property) => (property.clone(), "ASC".to_string()),
          bcr::pagination::Sort::DESC(property) => (property.clone(), "DESC".to_string()),
        };

        let property = match property.as_str() {
          "username" => "m.username".to_string(),
          "nickname" => "m.nickname".to_string(),
          "mobile_phone_number" => "m.mobile_phone_number".to_string(),
          "last_signined_ip" => "m.last_signined_ip".to_string(),
          "state" => "m.state".to_string(),
          "state_changed_at" => "m.state_changed_at".to_string(),
          "created_at" => "m.created_at".to_string(),
          "updated_at" => "m.updated_at".to_string(),
          "deleted_at" => "m.deleted_at".to_string(),
          _ => "".to_string(),
        };

        if !property.is_empty() {
          write!(&mut query_ordery_by, "{} {}", property, sort)
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
      }
    }

    Ok(query_ordery_by)
  }

  fn get_where(&self, find_all: &repositories::member::models::FindAll) -> Result<String, Error> {
    let mut query_where = String::new();

    if let Some(s) = &find_all.search {
      if let Some(sp) = s.site_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.site_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = s.member_class_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.member_class_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = s.member_level_id {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.member_level_id = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.username {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.username = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if !s.usernames.is_empty() {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }

        write!(
          &mut query_where,
          "m.username IN ({})",
          s.usernames
            .iter()
            .map(|v| format!("'{}'", v))
            .collect::<Vec<String>>()
            .join(",")
        )
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if let Some(sp) = &s.username_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.username like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.nickname {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.nickname = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if !s.nicknames.is_empty() {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }

        write!(
          &mut query_where,
          "m.nickname IN ({})",
          s.nicknames
            .iter()
            .map(|v| format!("'{}'", v))
            .collect::<Vec<String>>()
            .join(",")
        )
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.nickname_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.nickname like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.mobile_phone_number_like {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.mobile_phone_number like '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = &s.last_signined_ip {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.last_signined_ip = '{}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if let Some(sp) = s.state {
        if !query_where.is_empty() {
          write!(&mut query_where, " AND ")
            .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
        }
        write!(&mut query_where, "m.state = '{:?}'", sp)
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }

      if !query_where.is_empty() {
        write!(&mut query_where, " AND ")
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
      if s.deleted_at.is_some() {
        write!(&mut query_where, "m.deleted_at IS NOT NULL")
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      } else {
        write!(&mut query_where, "m.deleted_at IS NULL")
          .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
      }
    }
    Ok(query_where)
  }

  ///
  pub fn select_all_count(
    &self,
    conn: &diesel::PgConnection,
    find_all: &repositories::member::models::FindAll,
  ) -> Result<i64, Error> {
    use diesel::sql_types::BigInt;

    #[derive(QueryableByName)]
    struct Count {
      #[sql_type = "BigInt"]
      count: i64,
    }

    let mut query = String::new();
    write!(&mut query, "{}", MEMBER_COUNT_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let query_where = self.get_where(find_all)?;
    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    match sql_query(query).get_result::<Count>(conn) {
      Ok(m) => Ok(m.count),
      Err(e) => match e {
        diesel::result::Error::NotFound => Ok(0),
        _ => Err(e),
      },
    }
  }

  ///
  pub fn select_all(
    &self,
    conn: &diesel::PgConnection,
    find_all: &repositories::member::models::FindAll,
  ) -> Result<Vec<models::MemberModel>, Error> {
    let mut query = String::new();
    write!(&mut query, "{}", MEMBER_QUERY)
      .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;

    let query_where = self.get_where(find_all)?;
    if !query_where.is_empty() {
      write!(&mut query, " WHERE {}", query_where)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }
    let query_ordery_by = self.get_ordery_by(find_all)?;
    if !query_ordery_by.is_empty() {
      write!(&mut query, " ORDER BY {}", query_ordery_by)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }
    let query_pagination = self.get_pagination(find_all)?;
    if !query_pagination.is_empty() {
      write!(&mut query, " {} ", query_pagination)
        .map_err(|e| diesel::result::Error::QueryBuilderError(e.to_string().into()))?;
    }

    println!("query: {}", query);

    let _list = match sql_query(query).get_results::<models::_MemberModel>(conn) {
      Ok(m) => m,
      Err(e) => match e {
        diesel::result::Error::NotFound => {
          return Ok(vec![]);
        }
        _ => {
          return Err(e);
        }
      },
    };

    let mut list: Vec<models::MemberModel> = vec![];

    for _m in _list {
      list.push(models::from_member_model(
        &_m,
        self.select_parent(conn, _m.parent_member_id)?,
        self.select_children_count(conn, _m.id)?,
      ));
    }

    Ok(list)
  }
}
